@@ -3,14 +3,15 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003-2005 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003-2011,
+ * the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
@@ -24,29 +25,25 @@
  */
 
 #include "awk.h"
+#include "math.h"
 
-/* r_force_number --- force a value to be numeric */
+static int is_ieee_magic_val(const char *val);
+static AWKNUM get_ieee_magic_val(const char *val);
+
+/* force_number --- force a value to be numeric */
 
 AWKNUM
-r_force_number(register NODE *n)
+r_force_number(NODE *n)
 {
-	register char *cp;
-	register char *cpend;
+	char *cp;
+	char *cpend;
 	char save;
 	char *ptr;
 	unsigned int newflags;
 	extern double strtod();
 
-#ifdef GAWKDEBUG
-	if (n == NULL)
-		cant_happen();
-	if (n->type != Node_val)
-		cant_happen();
-	if (n->flags == 0)
-		cant_happen();
 	if (n->flags & NUMCUR)
 		return n->numbr;
-#endif
 
 	/* all the conditionals are an attempt to avoid the expensive strtod */
 
@@ -55,24 +52,44 @@ r_force_number(register NODE *n)
 	n->numbr = 0.0;
 
 	if (n->stlen == 0) {
-		if (0 && do_lint)
-			lintwarn(_("can't convert string to float"));
 		return 0.0;
 	}
 
 	cp = n->stptr;
-	if (ISALPHA(*cp)) {
-		if (0 && do_lint)
-			lintwarn(_("can't convert string to float"));
-		return 0.0;
+	/*
+	 * 2/2007:
+	 * POSIX, by way of severe language lawyering, seems to
+	 * allow things like "inf" and "nan" to mean something.
+	 * So if do_posix, the user gets what he deserves.
+	 * This also allows hexadecimal floating point. Ugh.
+	 */
+	if (! do_posix) {
+		if (isalpha((unsigned char) *cp)) {
+			return 0.0;
+		} else if (n->stlen == 4 && is_ieee_magic_val(n->stptr)) {
+			if (n->flags & MAYBE_NUM)
+				n->flags &= ~MAYBE_NUM;
+			n->flags |= NUMBER|NUMCUR;
+			n->numbr = get_ieee_magic_val(n->stptr);
+
+			return n->numbr;
+		}
+		/* else
+			fall through */
 	}
+	/* else not POSIX, so
+		fall through */
 
 	cpend = cp + n->stlen;
-	while (cp < cpend && ISSPACE(*cp))
+	while (cp < cpend && isspace((unsigned char) *cp))
 		cp++;
-	if (cp == cpend || ISALPHA(*cp)) {
-		if (0 && do_lint)
-			lintwarn(_("can't convert string to float"));
+
+	if (   cp == cpend		/* only spaces, or */
+	    || (! do_posix		/* not POSIXLY paranoid and */
+	        && (isalpha((unsigned char) *cp)	/* letter, or */
+					/* CANNOT do non-decimal and saw 0x */
+		    || (! do_non_decimal_data && cp[0] == '0'
+		        && (cp[1] == 'x' || cp[1] == 'X'))))) {
 		return 0.0;
 	}
 
@@ -81,21 +98,22 @@ r_force_number(register NODE *n)
 		n->flags &= ~MAYBE_NUM;
 	} else
 		newflags = 0;
-	if (cpend - cp == 1) {
-		if (ISDIGIT(*cp)) {
+
+	if (cpend - cp == 1) {		/* only one character */
+		if (isdigit((unsigned char) *cp)) {	/* it's a digit! */
 			n->numbr = (AWKNUM)(*cp - '0');
 			n->flags |= newflags;
 			n->flags |= NUMCUR;
-		} else if (0 && do_lint)
-			lintwarn(_("can't convert string to float"));
+		}
 		return n->numbr;
 	}
 
-	if (do_non_decimal_data) {
+	if (do_non_decimal_data) {	/* main.c assures false if do_posix */
 		errno = 0;
 		if (! do_traditional && isnondecimal(cp, TRUE)) {
 			n->numbr = nondec2awknum(cp, cpend - cp);
 			n->flags |= NUMCUR;
+			ptr = cpend;
 			goto finish;
 		}
 	}
@@ -106,29 +124,27 @@ r_force_number(register NODE *n)
 	n->numbr = (AWKNUM) strtod((const char *) cp, &ptr);
 
 	/* POSIX says trailing space is OK for NUMBER */
-	while (ISSPACE(*ptr))
+	while (isspace((unsigned char) *ptr))
 		ptr++;
 	*cpend = save;
 finish:
-	/* the >= should be ==, but for SunOS 3.5 strtod() */
-	if (errno == 0 && ptr >= cpend) {
+	if (errno == 0 && ptr == cpend) {
 		n->flags |= newflags;
 		n->flags |= NUMCUR;
 	} else {
-		if (0 && do_lint && ptr < cpend)
-			lintwarn(_("can't convert string to float"));
 		errno = 0;
 	}
 
 	return n->numbr;
 }
 
+
 /*
- * the following lookup table is used as an optimization in force_string
+ * The following lookup table is used as an optimization in force_string;
  * (more complicated) variations on this theme didn't seem to pay off, but 
- * systematic testing might be in order at some point
+ * systematic testing might be in order at some point.
  */
-static const char *const values[] = {
+static const char *values[] = {
 	"0",
 	"1",
 	"2",
@@ -145,10 +161,10 @@ static const char *const values[] = {
 /* format_val --- format a numeric value based on format */
 
 NODE *
-format_val(const char *format, int index, register NODE *s)
+format_val(const char *format, int index, NODE *s)
 {
 	char buf[BUFSIZ];
-	register char *sp = buf;
+	char *sp = buf;
 	double val;
 	char *orig, *trans, save;
 
@@ -160,15 +176,31 @@ format_val(const char *format, int index, register NODE *s)
 		trans = dgettext(TEXTDOMAIN, orig);
 
 		s->stptr[s->stlen] = save;
-		return tmp_string(trans, strlen(trans));
+		return make_string(trans, strlen(trans));
 	}
 
+	/*
+	 * 2/2007: Simplify our lives here. Instead of worrying about
+	 * whether or not the value will fit into a long just so we
+	 * can use sprintf("%ld", val) on it, always format it ourselves.
+	 * The only thing to worry about is that integral values always
+	 * format as integers. %.0f does that very well.
+	 *
+	 * 6/2008: Would that things were so simple. Always using %.0f
+	 * imposes a notable performance penalty for applications that
+	 * do a lot of conversion of integers to strings. So, we reinstate
+	 * the old code, but use %.0f for integral values that are outside
+	 * the range of a long.  This seems a reasonable compromise.
+	 *
+	 * 12/2009: Use <= and >= in the comparisons with LONG_xxx instead of
+	 * < and > so that things work correctly on systems with 64 bit integers.
+	 */
+
 	/* not an integral value, or out of range */
 	if ((val = double_to_int(s->numbr)) != s->numbr
-	    || val < LONG_MIN || val > LONG_MAX) {
+	    || val <= LONG_MIN || val >= LONG_MAX) {
 		/*
-		 * Once upon a time, if GFMT_WORKAROUND wasn't defined,
-		 * we just blindly did this:
+		 * Once upon a time, we just blindly did this:
 		 *	sprintf(sp, format, s->numbr);
 		 *	s->stlen = strlen(sp);
 		 *	s->stfmt = (char) index;
@@ -176,32 +208,37 @@ format_val(const char *format, int index, register NODE *s)
 		 * and just always format the value ourselves.
 		 */
 
-		NODE *dummy, *r;
+		NODE *dummy[2], *r;
 		unsigned short oflags;
 		extern NODE **fmt_list;          /* declared in eval.c */
 
 		/* create dummy node for a sole use of format_tree */
-		getnode(dummy);
-		dummy->type = Node_expression_list;
-		dummy->lnode = s;
-		dummy->rnode = NULL;
+		dummy[1] = s;
 		oflags = s->flags;
-		s->flags |= PERM; /* prevent from freeing by format_tree() */
-		r = format_tree(format, fmt_list[index]->stlen, dummy, 2);
+		if (val == s->numbr) {
+			/* integral value, but outside range of %ld, use %.0f */
+			r = format_tree("%.0f", 4, dummy, 2);
+			s->stfmt = -1;
+		} else {
+			r = format_tree(format, fmt_list[index]->stlen, dummy, 2);
+			assert(r != NULL);
+			s->stfmt = (char) index;
+		}
 		s->flags = oflags;
-		s->stfmt = (char) index;
 		s->stlen = r->stlen;
 		if ((s->flags & STRCUR) != 0)
-			free(s->stptr);
+			efree(s->stptr);
 		s->stptr = r->stptr;
-		freenode(r);		/* Do not free_temp(r)!  We want */
-		freenode(dummy);	/* to keep s->stptr == r->stpr.  */
+		freenode(r);	/* Do not unref(r)! We want to keep s->stptr == r->stpr.  */
 
 		goto no_malloc;
 	} else {
-		/* integral value */
-	        /* force conversion to long only once */
-		register long num = (long) val;
+		/*
+		 * integral value
+	         * force conversion to long only once
+		 */
+		long num = (long) val;
+
 		if (num < NVAL && num >= 0) {
 			sp = (char *) values[num];
 			s->stlen = 1;
@@ -211,176 +248,131 @@ format_val(const char *format, int index, register NODE *s)
 		}
 		s->stfmt = -1;
 	}
+	if (s->stptr != NULL)
+		efree(s->stptr);
 	emalloc(s->stptr, char *, s->stlen + 2, "format_val");
 	memcpy(s->stptr, sp, s->stlen+1);
 no_malloc:
-	s->stref = 1;
 	s->flags |= STRCUR;
-#if defined MBS_SUPPORT
-	if ((s->flags & WSTRCUR) != 0) {
-		assert(s->wstptr != NULL);
-		free(s->wstptr);
-		s->wstptr = NULL;
-		s->wstlen = 0;
-		s->flags &= ~WSTRCUR;
-	}
-#endif
+	free_wstr(s);
 	return s;
 }
 
-/* r_force_string --- force a value to be a string */
+/* force_string --- force a value to be a string */
 
 NODE *
-r_force_string(register NODE *s)
+r_force_string(NODE *s)
 {
-	NODE *ret;
-#ifdef GAWKDEBUG
-	if (s == NULL)
-		cant_happen();
-	if (s->type != Node_val)
-		cant_happen();
-	if (s->stref <= 0)
-		cant_happen();
 	if ((s->flags & STRCUR) != 0
-	    && (s->stfmt == -1 || s->stfmt == CONVFMTidx))
+		    && (s->stfmt == -1 || s->stfmt == CONVFMTidx)
+	)
 		return s;
-#endif
-
-	ret = format_val(CONVFMT, CONVFMTidx, s);
-	return ret;
+	return format_val(CONVFMT, CONVFMTidx, s);
 }
 
-/*
- * dupnode:
- * Duplicate a node.  (For strings, "duplicate" means crank up the
- * reference count.)
- */
+/* dupnode --- duplicate a node */
 
 NODE *
-r_dupnode(NODE *n)
+dupnode(NODE *n)
 {
-	register NODE *r;
+	NODE *r;
 
-#ifndef DUPNODE_MACRO
-	if ((n->flags & TEMP) != 0) {
-		n->flags &= ~TEMP;
-		n->flags |= MALLOC;
+	if (n->type == Node_ahash) {
+		n->ahname_ref++;
 		return n;
 	}
+
+	assert(n->type == Node_val);
+
 	if ((n->flags & PERM) != 0)
 		return n;
-#endif
-	if ((n->flags & (MALLOC|STRCUR)) == (MALLOC|STRCUR)) {
-		if (n->stref < LONG_MAX)
-			n->stref++;
-		else
-			n->flags |= PERM;
-		return n;
-	} else if ((n->flags & MALLOC) != 0 && n->type == Node_ahash) {
-		if (n->ahname_ref < LONG_MAX)
-			n->ahname_ref++;
-		else
-			n->flags |= PERM;
+	
+	if ((n->flags & MALLOC) != 0) {
+		n->valref++;
 		return n;
 	}
+
 	getnode(r);
 	*r = *n;
-	r->flags &= ~(PERM|TEMP|FIELD);
+	r->flags &= ~FIELD;
 	r->flags |= MALLOC;
-#if defined MBS_SUPPORT
+	r->valref = 1;
+#if MBS_SUPPORT
+	/*
+	 * DON'T call free_wstr(r) here!
+	 * r->wstptr still points at n->wstptr's value, and we
+	 * don't want to free it!
+	 */
 	r->wstptr = NULL;
-#endif /* defined MBS_SUPPORT */
-	if (n->type == Node_val && (n->flags & STRCUR) != 0) {
-		r->stref = 1;
-		emalloc(r->stptr, char *, r->stlen + 2, "dupnode");
-		memcpy(r->stptr, n->stptr, r->stlen);
-		r->stptr[r->stlen] = '\0';
-#if defined MBS_SUPPORT
+	r->wstlen = 0;
+#endif /* MBS_SUPPORT */
+
+	if ((n->flags & STRCUR) != 0) {
+		emalloc(r->stptr, char *, n->stlen + 2, "dupnode");
+		memcpy(r->stptr, n->stptr, n->stlen);
+		r->stptr[n->stlen] = '\0';
+#if MBS_SUPPORT
 		if ((n->flags & WSTRCUR) != 0) {
 			r->wstlen = n->wstlen;
-			emalloc(r->wstptr, wchar_t *, sizeof(wchar_t) * (r->wstlen + 2), "dupnode");
-			memcpy(r->wstptr, n->wstptr, r->wstlen * sizeof(wchar_t));
-			r->wstptr[r->wstlen] = L'\0';
+			emalloc(r->wstptr, wchar_t *, sizeof(wchar_t) * (n->wstlen + 2), "dupnode");
+			memcpy(r->wstptr, n->wstptr, n->wstlen * sizeof(wchar_t));
+			r->wstptr[n->wstlen] = L'\0';
 			r->flags |= WSTRCUR;
 		}
-#endif /* defined MBS_SUPPORT */
-	} else if (n->type == Node_ahash && (n->flags & MALLOC) != 0) {
-		r->ahname_ref = 1;
-		emalloc(r->ahname_str, char *, r->ahname_len + 2, "dupnode");
-		memcpy(r->ahname_str, n->ahname_str, r->ahname_len);
-		r->ahname_str[r->ahname_len] = '\0';
+#endif /* MBS_SUPPORT */
 	}
+	
 	return r;
 }
 
-/* copy_node --- force a brand new copy of a node to be allocated */
-
-NODE *
-copynode(NODE *old)
-{
-	NODE *new;
-	int saveflags;
-
-	assert(old != NULL);
-	saveflags = old->flags;
-	old->flags &= ~(MALLOC|PERM);
-	new = dupnode(old);
-	old->flags = saveflags;
-	return new;
-}
-
 /* mk_number --- allocate a node with defined number */
 
 NODE *
 mk_number(AWKNUM x, unsigned int flags)
 {
-	register NODE *r;
+	NODE *r;
 
 	getnode(r);
 	r->type = Node_val;
 	r->numbr = x;
+	r->valref = 1;
 	r->flags = flags;
-#ifdef GAWKDEBUG
-	r->stref = 1;
 	r->stptr = NULL;
 	r->stlen = 0;
-#if defined MBS_SUPPORT
-	r->wstptr = NULL;
-	r->wstlen = 0;
-	r->flags &= ~WSTRCUR;
-#endif /* MBS_SUPPORT */
-#endif /* GAWKDEBUG */
+	free_wstr(r);
 	return r;
 }
 
 /* make_str_node --- make a string node */
 
 NODE *
-make_str_node(char *s, unsigned long len, int flags)
+r_make_str_node(const char *s, unsigned long len, int flags)
 {
-	register NODE *r;
+	NODE *r;
 
 	getnode(r);
 	r->type = Node_val;
+	r->numbr = 0;
 	r->flags = (STRING|STRCUR|MALLOC);
-#if defined MBS_SUPPORT
+#if MBS_SUPPORT
 	r->wstptr = NULL;
 	r->wstlen = 0;
-#endif
+#endif /* MBS_SUPPORT */
+
 	if (flags & ALREADY_MALLOCED)
-		r->stptr = s;
+		r->stptr = (char *) s;
 	else {
-		emalloc(r->stptr, char *, len + 2, s);
+		emalloc(r->stptr, char *, len + 2, "make_str_node");
 		memcpy(r->stptr, s, len);
 	}
 	r->stptr[len] = '\0';
-	       
+
 	if ((flags & SCAN) != 0) {	/* scan for escape sequences */
 		const char *pf;
-		register char *ptm;
-		register int c;
-		register const char *end;
-#ifdef MBS_SUPPORT
+		char *ptm;
+		int c;
+		const char *end;
+#if MBS_SUPPORT
 		mbstate_t cur_state;
 
 		memset(& cur_state, 0, sizeof(cur_state));
@@ -388,7 +380,7 @@ make_str_node(char *s, unsigned long len, int flags)
 
 		end = &(r->stptr[len]);
 		for (pf = ptm = r->stptr; pf < end;) {
-#ifdef MBS_SUPPORT
+#if MBS_SUPPORT
 			/*
 			 * Keep multibyte characters together. This avoids
 			 * problems if a subsequent byte of a multibyte
@@ -421,27 +413,16 @@ make_str_node(char *s, unsigned long len, int flags)
 		len = ptm - r->stptr;
 		erealloc(r->stptr, char *, len + 1, "make_str_node");
 		r->stptr[len] = '\0';
+		r->flags &= ~MALLOC;
 		r->flags |= PERM;
 	}
 	r->stlen = len;
-	r->stref = 1;
+	r->valref = 1;
 	r->stfmt = -1;
 
 	return r;
 }
 
-/* tmp_string --- allocate a temporary string */
-
-NODE *
-tmp_string(char *s, size_t len)
-{
-	register NODE *r;
-
-	r = make_string(s, len);
-	r->flags |= TEMP;
-	return r;
-}
-
 /* more_nodes --- allocate more nodes */
 
 #define NODECHUNK	100
@@ -451,7 +432,7 @@ NODE *nextfree = NULL;
 NODE *
 more_nodes()
 {
-	register NODE *np;
+	NODE *np;
 
 	/* get more nodes and initialize list */
 	emalloc(nextfree, NODE *, NODECHUNK * sizeof(NODE), "more_nodes");
@@ -466,69 +447,39 @@ more_nodes()
 	return np;
 }
 
-#ifdef MEMDEBUG
-#undef freenode
-/* freenode --- release a node back to the pool */
-
-void
-freenode(NODE *it)
-{
-#ifdef MPROF
-	it->stref = 0;
-	free((char *) it);
-#else	/* not MPROF */
-#ifndef NO_PROFILING
-	it->exec_count = 0;
-#endif
-	/* add it to head of freelist */
-	it->nextp = nextfree;
-	nextfree = it;
-#endif	/* not MPROF */
-}
-#endif	/* GAWKDEBUG */
-
 /* unref --- remove reference to a particular node */
 
 void
-unref(register NODE *tmp)
+unref(NODE *tmp)
 {
 	if (tmp == NULL)
 		return;
 	if ((tmp->flags & PERM) != 0)
 		return;
-	tmp->flags &= ~TEMP;
-	if ((tmp->flags & MALLOC) != 0) {
-		if (tmp->type == Node_ahash) {
-			if (tmp->ahname_ref > 1) {
-				tmp->ahname_ref--;
-				return;
-			}
-			free(tmp->ahname_str);
-		} else if ((tmp->flags & STRCUR) != 0) {
-			if (tmp->stref > 1) {
-				tmp->stref--;
-				return;
-			}
-			free(tmp->stptr);
-#if defined MBS_SUPPORT
-			if (tmp->wstptr != NULL) {
-				assert((tmp->flags & WSTRCUR) != 0);
-				free(tmp->wstptr);
-			}
-			tmp->flags &= ~WSTRCUR;
-			tmp->wstptr = NULL;
-			tmp->wstlen = 0;
-#endif
+
+	if (tmp->type == Node_ahash) {
+		if (tmp->ahname_ref > 1)
+			tmp->ahname_ref--;
+		else {
+			efree(tmp->ahname_str);
+			freenode(tmp);
 		}
-		freenode(tmp);
 		return;
 	}
-	if ((tmp->flags & FIELD) != 0) {
-		freenode(tmp);
-		return;
+
+	if ((tmp->flags & MALLOC) != 0) {
+		if (tmp->valref > 1) {
+			tmp->valref--;  
+			return;
+		} 
+		if (tmp->flags & STRCUR)
+			efree(tmp->stptr);
 	}
+	free_wstr(tmp);
+	freenode(tmp);
 }
 
+
 /*
  * parse_escape:
  *
@@ -545,19 +496,32 @@ unref(register NODE *tmp)
  * If \ is followed by 000, we return 0 and leave the string pointer after the
  * zeros.  A value of 0 does not mean end of string.  
  *
- * Posix doesn't allow \x.
+ * POSIX doesn't allow \x.
  */
 
 int
 parse_escape(const char **string_ptr)
 {
-	register int c = *(*string_ptr)++;
-	register int i;
-	register int count;
+	int c = *(*string_ptr)++;
+	int i;
+	int count;
+	int j;
+	const char *start;
+
+	if (do_lint_old) {
+		switch (c) {
+		case 'a':
+		case 'b':
+		case 'f':
+		case 'r':
+			warning(_("old awk does not support the `\\%c' escape sequence"), c);
+			break;
+		}
+	}
 
 	switch (c) {
 	case 'a':
-		return BELL;
+		return '\a';
 	case 'b':
 		return '\b';
 	case 'f':
@@ -597,28 +561,29 @@ parse_escape(const char **string_ptr)
 		return i;
 	case 'x':
 		if (do_lint) {
-			static int didwarn = FALSE;
+			static short warned = FALSE;
 
-			if (! didwarn) {
-				didwarn = TRUE;
+			if (! warned) {
+				warned = TRUE;
 				lintwarn(_("POSIX does not allow `\\x' escapes"));
 			}
 		}
 		if (do_posix)
 			return ('x');
-		if (! ISXDIGIT((*string_ptr)[0])) {
+		if (! isxdigit((unsigned char) (*string_ptr)[0])) {
 			warning(_("no hex digits in `\\x' escape sequence"));
 			return ('x');
 		}
-		i = 0;
-		for (;;) {
+		i = j = 0;
+		start = *string_ptr;
+		for (;; j++) {
 			/* do outside test to avoid multiple side effects */
 			c = *(*string_ptr)++;
-			if (ISXDIGIT(c)) {
+			if (isxdigit(c)) {
 				i *= 16;
-				if (ISDIGIT(c))
+				if (isdigit(c))
 					i += c - '0';
-				else if (ISUPPER(c))
+				else if (isupper(c))
 					i += c - 'A' + 10;
 				else
 					i += c - 'a' + 10;
@@ -627,6 +592,8 @@ parse_escape(const char **string_ptr)
 				break;
 			}
 		}
+		if (do_lint && j > 2)
+			lintwarn(_("hex escape \\x%.*s of %d characters probably not interpreted the way you expect"), j, start, j);
 		return i;
 	case '\\':
 	case '"':
@@ -679,14 +646,14 @@ isnondecimal(const char *str, int use_locale)
 	for (; *str != '\0'; str++) {
 		if (*str == 'e' || *str == 'E' || *str == dec_point)
 			return FALSE;
-		else if (! ISDIGIT(*str))
+		else if (! isdigit((unsigned char) *str))
 			break;
 	}
 
 	return TRUE;
 }
 
-#if defined MBS_SUPPORT
+#if MBS_SUPPORT
 /* str2wstr --- convert a multibyte string to a wide string */
 
 NODE *
@@ -696,20 +663,25 @@ str2wstr(NODE *n, size_t **ptr)
 	char *sp;
 	mbstate_t mbs;
 	wchar_t wc, *wsp;
+	static short warned = FALSE;
 
 	assert((n->flags & (STRING|STRCUR)) != 0);
 
+	/*
+	 * Don't convert global null string or global null field
+	 * variables to a wide string. They are both zero-length anyway.
+	 * This also avoids future double-free errors while releasing
+	 * shallow copies, eg. *tmp = *Null_field; free_wstr(tmp);
+	 */
+	if (n == Nnull_string || n == Null_field)
+		return n;
+
 	if ((n->flags & WSTRCUR) != 0) {
 		if (ptr == NULL)
 			return n;
 		/* otherwise
 			fall through and recompute to fill in the array */
-	}
-
-	if (n->wstptr != NULL) {
-		free(n->wstptr);
-		n->wstptr = NULL;
-		n->wstlen = 0;
+		free_wstr(n);
 	}
 
 	/*
@@ -717,11 +689,11 @@ str2wstr(NODE *n, size_t **ptr)
 	 * code trades space for time. We allocate
 	 * an array of wchar_t that is n->stlen long.
 	 * This is needed in the worst case anyway, where
-	 * each input bytes maps to one wchar_t.  The
+	 * each input byte maps to one wchar_t.  The
 	 * advantage is that we only have to convert the string
 	 * once, instead of twice, once to find out how many
-	 * wide characters, and then again to actually fill
-	 * the info in.  If there's a lot left over, we can
+	 * wide characters, and then again to actually fill in
+	 * the info.  If there's a lot left over, we can
 	 * realloc the wide string down in size.
 	 */
 
@@ -745,13 +717,43 @@ str2wstr(NODE *n, size_t **ptr)
 	src_count = n->stlen;
 	memset(& mbs, 0, sizeof(mbs));
 	for (i = 0; src_count > 0; i++) {
-		count = mbrtowc(& wc, sp, src_count, & mbs);
+		/*
+		 * 9/2010: Check the current byte; if it's a valid character,
+		 * then it doesn't start a multibyte sequence. This brings a
+		 * big speed up. Thanks to Ulrich Drepper for the tip.
+		 * 11/2010: Thanks to Paolo Bonzini for some even faster code.
+		 */
+		if (is_valid_character(*sp)) {
+			count = 1;
+			wc = btowc_cache(*sp);
+		} else
+			count = mbrtowc(& wc, sp, src_count, & mbs);
 		switch (count) {
 		case (size_t) -2:
 		case (size_t) -1:
-		case 0:
-			goto done;
+			/*
+			 * Just skip the bad byte and keep going, so that
+			 * we get a more-or-less full string, instead of
+			 * stopping early. This is particularly important
+			 * for match() where we need to build the indices.
+			 */
+			sp++;
+			src_count--;
+			/*
+			 * mbrtowc(3) says the state of mbs becomes undefined
+			 * after a bad character, so reset it.
+			 */
+			memset(& mbs, 0, sizeof(mbs));
+			/* And warn the user something's wrong */
+			if (do_lint && ! warned) {
+				warned = TRUE;
+				lintwarn(_("Invalid multibyte data detected. There may be a mismatch between your data and your locale."));
+			}
+			break;
 
+		case 0:
+			count = 1;
+			/* fall through */
 		default:
 			*wsp++ = wc;
 			src_count -= count;
@@ -764,9 +766,8 @@ str2wstr(NODE *n, size_t **ptr)
 		}
 	}
 
-done:
 	*wsp = L'\0';
-	n->wstlen = i;
+	n->wstlen = wsp - n->wstptr;
 	n->flags |= WSTRCUR;
 #define ARBITRARY_AMOUNT_TO_GIVE_BACK 100
 	if (n->stlen - n->wstlen > ARBITRARY_AMOUNT_TO_GIVE_BACK)
@@ -775,17 +776,72 @@ done:
 	return n;
 }
 
-#if 0
-static void
+/* wstr2str --- convert a wide string back into multibyte one */
+
+NODE *
+wstr2str(NODE *n)
+{
+	size_t result;
+	size_t length;
+	wchar_t *wp;
+	mbstate_t mbs;
+	char *newval, *cp;
+
+	assert(n->valref == 1);
+	assert((n->flags & WSTRCUR) != 0);
+
+	/*
+	 * Convert the wide chars in t1->wstptr back into m.b. chars.
+	 * This is pretty grotty, but it's the most straightforward
+	 * way to do things.
+	 */
+	memset(& mbs, 0, sizeof(mbs));
+
+	length = n->wstlen;
+	emalloc(newval, char *, (length * gawk_mb_cur_max) + 2, "wstr2str");
+
+	wp = n->wstptr;
+	for (cp = newval; length > 0; length--) {
+		result = wcrtomb(cp, *wp, & mbs);
+		if (result == (size_t) -1)	/* what to do? break seems best */
+			break;
+		cp += result;
+		wp++;
+	}
+	*cp = '\0';
+
+	efree(n->stptr);
+	n->stptr = newval;
+	n->stlen = cp - newval;
+
+	return n;
+}
+
+/* free_wstr --- release the wide string part of a node */
+
+void
+free_wstr(NODE *n)
+{
+	assert(n->type == Node_val);
+
+	if ((n->flags & WSTRCUR) != 0) {
+		assert(n->wstptr != NULL);
+		efree(n->wstptr);
+	}
+	n->wstptr = NULL;
+	n->wstlen = 0;
+	n->flags &= ~WSTRCUR;
+}
+
+static void __attribute__ ((unused))
 dump_wstr(FILE *fp, const wchar_t *str, size_t len)
 {
 	if (str == NULL || len == 0)
 		return;
 
 	for (; len--; str++)
-		putc((int) *str, fp);
+		putwc(*str, fp);
 }
-#endif
 
 /* wstrstr --- walk haystack, looking for needle, wide char version */
 
@@ -846,4 +902,64 @@ out:	;
 
 	return NULL;
 }
-#endif /* defined MBS_SUPPORT */
+#endif /* MBS_SUPPORT */
+
+/* is_ieee_magic_val --- return true for +inf, -inf, +nan, -nan */
+
+static int
+is_ieee_magic_val(const char *val)
+{
+	/*
+	 * Avoid strncasecmp: it mishandles ASCII bytes in some locales.
+	 * Assume the length is 4, as the caller checks this.
+	 */
+	return (   (val[0] == '+' || val[0] == '-')
+		&& (   (   (val[1] == 'i' || val[1] == 'I')
+			&& (val[2] == 'n' || val[2] == 'N')
+			&& (val[3] == 'f' || val[3] == 'F'))
+		    || (   (val[1] == 'n' || val[1] == 'N')
+			&& (val[2] == 'a' || val[2] == 'A')
+			&& (val[3] == 'n' || val[3] == 'N'))));
+}
+
+/* get_ieee_magic_val --- return magic value for string */
+
+static AWKNUM
+get_ieee_magic_val(const char *val)
+{
+	static short first = TRUE;
+	static AWKNUM inf;
+	static AWKNUM nan;
+
+	char *ptr;
+	AWKNUM v = strtod(val, &ptr);
+
+	if (val == ptr) { /* Older strtod implementations don't support inf or nan. */
+		if (first) {
+			first = FALSE;
+			nan = sqrt(-1.0);
+			inf = -log(0.0);
+		}
+
+		v = ((val[1] == 'i' || val[1] == 'I') ? inf : nan);
+		if (val[0] == '-')
+			v = -v;
+	}
+
+	return v;
+}
+
+#if MBS_SUPPORT
+wint_t btowc_cache[256];
+
+/* init_btowc_cache --- initialize the cache */
+
+void init_btowc_cache()
+{
+	int i;
+
+	for (i = 0; i < 255; i++) {
+		btowc_cache[i] = btowc(i);
+	}
+}
+#endif
